@@ -10,9 +10,11 @@ use SPVM::Builder;
 use SPVM::Builder::CC;
 use SPVM::Builder::Util;
 use SPVM::Builder::Config::Exe;
-use SPVM::Builder::Runtime;
 use JSON::PP;
 
+use SPVM 'Native::Compiler';
+use SPVM 'Native::Runtime';
+
 use File::Spec;
 use File::Find 'find';
 
@@ -196,10 +198,8 @@ sub new {
   
   $self->{builder} = $builder;
   
-  # Compile SPVM
-  my $compiler = SPVM::Builder::Compiler->new(
-    include_dirs => $builder->include_dirs
-  );
+  my $compiler = SPVM::Native::Compiler->new;
+  $compiler->add_include_dir($_) for @{$builder->include_dirs};
   $self->{compiler} = $compiler;
   
   return bless $self, $class;
@@ -211,7 +211,6 @@ sub build_exe_file {
   # Builder
   my $builder = $self->builder;
   
-  # Module name
   my $basic_type_name = $self->{module_name};
   
   # Build runtime
@@ -219,12 +218,13 @@ sub build_exe_file {
     $self->compile;
   }
   
-  # Config file
-  my $module_file = $self->runtime->get_module_file($basic_type_name);
-
+  my $basic_type = $self->runtime->get_basic_type_by_name($basic_type_name);
+  
+  my $module_file = $basic_type->_get_module_file;
+  
   # Object files
   my $object_files = [];
-
+  
   # Compile SPVM core source files
   my $spvm_core_object_files = $self->compile_spvm_core_source_files;
   push @$object_files, @$spvm_core_object_files;
@@ -234,7 +234,7 @@ sub build_exe_file {
   
   # Create bootstrap C source
   $self->create_bootstrap_source;
-
+  
   # Compile bootstrap C source
   my $bootstrap_object_file = $self->compile_bootstrap_source_file;
   push @$object_files, $bootstrap_object_file;
@@ -242,7 +242,7 @@ sub build_exe_file {
   # Build directory
   my $build_dir = $self->builder->build_dir;
   mkpath $build_dir;
-
+  
   # Link and generate executable file
   my $config_exe = $self->config;
   my $cc_linker = SPVM::Builder::CC->new(
@@ -255,7 +255,7 @@ sub build_exe_file {
     config => $self->config,
     category => 'native',
   };
-
+  
   $cc_linker->link($basic_type_name, $object_files, $options);
 }
 
@@ -281,25 +281,25 @@ sub get_required_resources {
     force => $self->force,
   );
   
-  my $basic_type_names = $self->get_module_names;
+  my $basic_type_names = $self->runtime->_get_user_defined_basic_type_names;
   my $all_object_files = [];
   for my $basic_type_name (@$basic_type_names) {
-
+    
+    my $basic_type = $self->runtime->get_basic_type_by_name($basic_type_name);
+    
     my $perl_basic_type_name = "SPVM::$basic_type_name";
     
-    my $native_method_names = $self->runtime->get_method_names($basic_type_name, 'native');
+    my $native_method_names = $basic_type->_get_native_method_names;
     if (@$native_method_names) {
-      my $native_module_file = $self->runtime->get_module_file($basic_type_name);
-      my $native_dir = $native_module_file;
+      my $module_file = $basic_type->_get_module_file;
+      my $native_dir = $module_file;
       
       $native_dir =~ s/\.spvm$//;
       $native_dir .= 'native';
-      my $input_dir = SPVM::Builder::Util::remove_basic_type_name_part_from_file($native_module_file, $perl_basic_type_name);
+      my $input_dir = SPVM::Builder::Util::remove_basic_type_name_part_from_file($module_file, $perl_basic_type_name);
       my $build_object_dir = SPVM::Builder::Util::create_build_object_path($self->builder->build_dir);
       mkpath $build_object_dir;
-
-      # Class file
-      my $module_file = $self->runtime->get_module_file($basic_type_name);
+      
       unless (defined $module_file) {
         my $config_exe_file = SPVM::Builder::Util::get_config_file_from_basic_type_name($basic_type_name);
         if ($config_exe_file) {
@@ -375,16 +375,18 @@ sub compile {
   
   my $compiler = $self->compiler;
   
-  my $success = $compiler->compile($basic_type_name, __FILE__, __LINE__);
+  $compiler->set_start_file(__FILE__);
+  $compiler->set_start_line(__LINE__ + 1);
+  my $success = $compiler->compile($basic_type_name);
   unless ($success) {
-    $compiler->print_error_messages(*STDERR);
+    my $error_messages = $compiler->get_error_messages;
+    for my $error_message (@$error_messages) {
+      print STDERR "$error_message\n";
+    }
     exit(255);
   }
   my $runtime = $compiler->get_runtime;
   
-  # This is not needed, but check when get_runtime is called more than one.
-  $runtime = $compiler->get_runtime;
-  
   $self->runtime($runtime);
   
   $self->{finish_compile} = 1;
@@ -393,7 +395,7 @@ sub compile {
 sub compile_modules {
   my ($self) = @_;
 
-  my $basic_type_names = $self->get_module_names;
+  my $basic_type_names = $self->runtime->_get_user_defined_basic_type_names;
   
   my $object_files = [];
   for my $basic_type_name (@$basic_type_names) {
@@ -494,7 +496,7 @@ sub create_bootstrap_header_source {
   my $basic_type_name = $self->module_name;
 
   # Module names
-  my $basic_type_names = $self->get_module_names;
+  my $basic_type_names = $self->runtime->_get_user_defined_basic_type_names;
   
   my $source = '';
   
@@ -516,7 +518,8 @@ EOS
   
   $source .= "// precompile functions declaration\n";
   for my $basic_type_name (@$basic_type_names) {
-    my $precompile_method_names = $self->runtime->get_method_names($basic_type_name, 'precompile');
+    my $basic_type = $self->runtime->get_basic_type_by_name($basic_type_name);
+    my $precompile_method_names = $basic_type->_get_precompile_method_names;
     for my $method_name (@$precompile_method_names) {
       my $method_cname = $basic_type_name;
       $method_cname =~ s/::/__/g;
@@ -537,13 +540,14 @@ env->api->method->set_precompile_address(env->runtime, method, precompile_addres
 EOS
   
   $source .= "// native functions declaration\n";
-  for my $method_cname (@$basic_type_names) {
-    my $native_method_names = $self->runtime->get_method_names($method_cname, 'native');
+  for my $basic_type_name (@$basic_type_names) {
+    my $basic_type = $self->runtime->get_basic_type_by_name($basic_type_name);
+    my $native_method_names = $basic_type->_get_native_method_names;
     for my $method_name (@$native_method_names) {
-      my $method_cname = $method_cname;
-      $method_cname =~ s/::/__/g;
+      my $basic_type_name = $basic_type_name;
+      $basic_type_name =~ s/::/__/g;
       $source .= <<"EOS";
-int32_t SPVM__${method_cname}__$method_name(SPVM_ENV* env, SPVM_VALUE* stack);
+int32_t SPVM__${basic_type_name}__$method_name(SPVM_ENV* env, SPVM_VALUE* stack);
 EOS
     }
   }
@@ -573,7 +577,7 @@ sub create_bootstrap_main_func_source {
   my $basic_type_name = $self->module_name;
 
   # Module names
-  my $basic_type_names = $self->get_module_names;
+  my $basic_type_names = $self->runtime->_get_user_defined_basic_type_names;
 
   my $source = '';
 
@@ -710,27 +714,35 @@ static void* SPVM_BOOTSTRAP_get_runtime(SPVM_ENV* env, void* compiler) {
   
 EOS
   
-  my $basic_type_names = $self->get_module_names;
+  my $basic_type_names = $self->runtime->_get_user_defined_basic_type_names;
   
   my $compiler = $self->compiler;
   
   for my $basic_type_name (@$basic_type_names) {
+    my $basic_type = $self->runtime->get_basic_type_by_name($basic_type_name);
+    
     my $module_file = $compiler->get_module_file($basic_type_name);
     
+    my $module_file_rel_file = $module_file->get_rel_file;
+    
+    my $module_file_content = $module_file->get_content;
+    
+    my $module_file_content_length = $module_file->get_content_length;
+    
     my $source_module_file = '';
     
     $source_module_file .= qq|  {\n|;
-
+    
     $source_module_file .= qq|    void* module_file = env->api->module_file->new_instance(compiler);\n|;
     
     $source_module_file .= qq|    env->api->module_file->set_module_name(compiler, module_file, "$basic_type_name");\n|;
     
-    if (defined $module_file->{rel_file}) {
-      $source_module_file .= qq|    env->api->module_file->set_rel_file(compiler, module_file, "$module_file->{rel_file}");\n|;
+    if (defined $module_file_rel_file) {
+      $source_module_file .= qq|    env->api->module_file->set_rel_file(compiler, module_file, "$module_file_rel_file");\n|;
     }
     
-    if (defined $module_file->{content}) {
-      my $content_espcaped = $module_file->{content};
+    if (defined $module_file_content) {
+      my $content_espcaped = $module_file_content;
       
       {
         use bytes;
@@ -742,9 +754,9 @@ EOS
       $source_module_file .= qq|    env->api->module_file->set_content(compiler, module_file, "$content_espcaped");\n|;
     }
     
-    $source_module_file .= qq|    env->api->module_file->set_content_length(compiler, module_file, $module_file->{content_length});\n|;
+    $source_module_file .= qq|    env->api->module_file->set_content_length(compiler, module_file, $module_file_content_length);\n|;
       
-    $source_module_file .= qq|    env->api->compiler->add_module_file(compiler, "$basic_type_name", module_file);\n\n|;
+    $source_module_file .= qq|    env->api->compiler->set_module_file(compiler, "$basic_type_name", module_file);\n\n|;
     
     $source_module_file .= qq|  }\n|;
     
@@ -782,17 +794,19 @@ sub create_bootstrap_set_precompile_method_addresses_func_source {
   my $builder = $self->builder;
 
   # Module names
-  my $basic_type_names = $self->get_module_names;
+  my $basic_type_names = $self->runtime->_get_user_defined_basic_type_names;
 
   my $source = '';
 
   $source .= "static int32_t* SPVM_BOOTSTRAP_create_bootstrap_set_precompile_method_addresses(SPVM_ENV* env){\n";
 
   for my $basic_type_name (@$basic_type_names) {
+    my $basic_type = $self->runtime->get_basic_type_by_name($basic_type_name);
+    
     my $method_cname = $basic_type_name;
     $method_cname =~ s/::/__/g;
     
-    my $precompile_method_names = $self->runtime->get_method_names($basic_type_name, 'precompile');
+    my $precompile_method_names = $basic_type->_get_precompile_method_names;
     
     for my $precompile_method_name (@$precompile_method_names) {
       $source .= <<"EOS";
@@ -813,17 +827,19 @@ sub create_bootstrap_set_native_method_addresses_func_source {
   my $builder = $self->builder;
 
   # Module names
-  my $basic_type_names = $self->get_module_names;
+  my $basic_type_names = $self->runtime->_get_user_defined_basic_type_names;
 
   my $source = '';
 
   $source .= "static int32_t* SPVM_BOOTSTRAP_create_bootstrap_set_native_method_addresses(SPVM_ENV* env){\n";
 
   for my $basic_type_name (@$basic_type_names) {
+    my $basic_type = $self->runtime->get_basic_type_by_name($basic_type_name);
+    
     my $method_cname = $basic_type_name;
     $method_cname =~ s/::/__/g;
     
-    my $native_method_names = $self->runtime->get_method_names($basic_type_name, 'native');
+    my $native_method_names = $basic_type->_get_native_method_names;
     
     for my $native_method_name (@$native_method_names) {
       $source .= <<"EOS";
@@ -847,13 +863,15 @@ sub create_bootstrap_source {
   my $basic_type_name = $self->module_name;
   
   # Module names
-  my $basic_type_names = $self->get_module_names;
+  my $basic_type_names = $self->runtime->_get_user_defined_basic_type_names;
   
-  # Class files - Input
   my $module_files = [];
   for my $basic_type_name (@$basic_type_names) {
-    my $module_file = $self->runtime->get_module_file($basic_type_name);
-    push @$module_files, $module_file;
+    my $basic_type = $self->runtime->get_basic_type_by_name($basic_type_name);
+    if ($basic_type->get_module_dir) {
+      my $module_file = $basic_type->_get_module_file;
+      push @$module_files, $module_file;
+    }
   }
   
   # Source file - Output
@@ -999,13 +1017,14 @@ sub compile_module_precompile_source_file {
   );
   
   my $object_files = [];
-  my $precompile_method_names = $self->runtime->get_method_names($basic_type_name, 'precompile');
+  my $basic_type = $self->runtime->get_basic_type_by_name($basic_type_name);
+  my $precompile_method_names = $basic_type->_get_precompile_method_names;
   if (@$precompile_method_names) {
     my $build_src_dir = SPVM::Builder::Util::create_build_src_path($self->builder->build_dir);
     mkpath $build_src_dir;
     
-    my $module_file = $self->runtime->get_module_file($basic_type_name);
-    my $precompile_source = $self->runtime->build_precompile_module_source($basic_type_name);
+    my $module_file = $basic_type->_get_module_file;
+    my $precompile_source = $self->runtime->build_precompile_module_source($basic_type);
     
     $builder_cc->build_precompile_module_source_file(
       $basic_type_name,
@@ -1059,19 +1078,19 @@ sub compile_module_native_source_files {
   
   my $perl_basic_type_name = "SPVM::$basic_type_name";
   
-  my $native_method_names = $self->runtime->get_method_names($basic_type_name, 'native');
+  my $basic_type = $self->runtime->get_basic_type_by_name($basic_type_name);
+  
+  my $native_method_names = $basic_type->_get_native_method_names;
   if (@$native_method_names) {
-    my $native_module_file = $self->runtime->get_module_file($basic_type_name);
-    my $native_dir = $native_module_file;
+    my $module_file = $basic_type->_get_module_file;
+    my $native_dir = $module_file;
     
     $native_dir =~ s/\.spvm$//;
     $native_dir .= 'native';
-    my $input_dir = SPVM::Builder::Util::remove_basic_type_name_part_from_file($native_module_file, $perl_basic_type_name);
+    my $input_dir = SPVM::Builder::Util::remove_basic_type_name_part_from_file($module_file, $perl_basic_type_name);
     my $build_object_dir = SPVM::Builder::Util::create_build_object_path($self->builder->build_dir);
     mkpath $build_object_dir;
 
-    # Class file
-    my $module_file = $self->runtime->get_module_file($basic_type_name);
     unless (defined $module_file) {
       my $config_file = SPVM::Builder::Util::get_config_file_from_basic_type_name($basic_type_name);
       if ($config_file) {
@@ -1112,16 +1131,6 @@ sub compile_module_native_source_files {
   return $all_object_files;
 }
 
-sub get_module_names {
-  my ($self) = @_;
-  
-  my $basic_type_names = $self->runtime->get_basic_type_names;
-  
-  $basic_type_names = [grep { /^[A-Z]/ && $_ !~ /::anon::/ } @$basic_type_names];
-  
-  return $basic_type_names;
-}
-
 1;
 
 =head1 Name
