@@ -53,26 +53,154 @@
 SPVM_COMPILER* SPVM_COMPILER_new() {
   SPVM_COMPILER* compiler = SPVM_ALLOCATOR_alloc_memory_block_unmanaged(sizeof(SPVM_COMPILER));
   
-  // Allocator
-  SPVM_ALLOCATOR* allocator = SPVM_ALLOCATOR_new();
-  compiler->allocator = allocator;
+  compiler->global_allocator = SPVM_ALLOCATOR_new();
+  compiler->error_message_allocator = SPVM_ALLOCATOR_new();
   
   compiler->ch_ptr = "";
   
-  compiler->constant_string_symtable = SPVM_HASH_new_hash_permanent(compiler->allocator, 128);
+  compiler->constant_string_symtable = SPVM_HASH_new_hash_permanent(compiler->global_allocator, 128);
+  
+  compiler->include_dirs = SPVM_LIST_new_list_permanent(compiler->global_allocator, 0);
+  compiler->basic_types = SPVM_LIST_new_list_permanent(compiler->global_allocator, 0);
+  compiler->basic_type_symtable = SPVM_HASH_new_hash_permanent(compiler->global_allocator, 0);
+  compiler->module_file_symtable = SPVM_HASH_new_hash_permanent(compiler->global_allocator, 0);
+  compiler->if_require_not_found_basic_type_name_symtable = SPVM_HASH_new_hash_permanent(compiler->global_allocator, 0);
+  compiler->each_compile_allocators = SPVM_LIST_new_list_permanent(compiler->global_allocator, 0);
+  compiler->error_messages = SPVM_LIST_new_list_permanent(compiler->global_allocator, 0);
+  
+  compiler->ops = SPVM_LIST_new_list_permanent(compiler->global_allocator, 0);
+  compiler->op_use_stack = SPVM_LIST_new_list_permanent(compiler->global_allocator, 0);
+  compiler->op_types = SPVM_LIST_new_list_permanent(compiler->global_allocator, 0);
   
-  // Eternal information
-  compiler->include_dirs = SPVM_LIST_new_list_permanent(compiler->allocator, 0);
-  compiler->basic_types = SPVM_LIST_new_list_permanent(compiler->allocator, 0);
-  compiler->basic_type_symtable = SPVM_HASH_new_hash_permanent(compiler->allocator, 0);
-  compiler->module_file_symtable = SPVM_HASH_new_hash_permanent(compiler->allocator, 0);
-  compiler->if_require_not_found_basic_type_name_symtable = SPVM_HASH_new_hash_permanent(compiler->allocator, 0);
   
   compiler->runtime = SPVM_RUNTIME_new();
   
   return compiler;
 }
 
+void SPVM_COMPILER_free(SPVM_COMPILER* compiler) {
+  
+  SPVM_COMPILER_clear_error_messages(compiler);
+  
+  const char* start_file = SPVM_COMPILER_get_start_file(compiler);
+  
+  if (start_file) {
+    SPVM_ALLOCATOR_free_memory_block_tmp(compiler->global_allocator, (void*)start_file);
+  }
+  
+  SPVM_COMPILER_clear_include_dirs(compiler);
+  
+  if (compiler->runtime) {
+    SPVM_RUNTIME_free(compiler->runtime);
+    compiler->runtime = NULL;
+  }
+  
+  for (int32_t i = 0; i < compiler->each_compile_allocators->length; i++) {
+    SPVM_ALLOCATOR* each_compile_allocator = SPVM_LIST_get(compiler->each_compile_allocators, i);
+    SPVM_ALLOCATOR_free(each_compile_allocator);
+  }
+  
+  SPVM_ALLOCATOR_free(compiler->error_message_allocator);
+  compiler->error_message_allocator = NULL;
+  
+  SPVM_ALLOCATOR_free(compiler->global_allocator);
+  compiler->global_allocator = NULL;
+}
+
+int32_t SPVM_COMPILER_compile(SPVM_COMPILER* compiler, const char* basic_type_name) {
+  
+  compiler->current_each_compile_allocator = SPVM_ALLOCATOR_new();
+  
+  SPVM_COMPILER_clear_error_messages(compiler);
+  
+  int32_t compile_start_memory_blocks_count_tmp = compiler->current_each_compile_allocator->memory_blocks_count_tmp;
+  
+  compiler->basic_types_base_id = compiler->basic_types->length;
+  
+  if (compiler->basic_types->length == 0) {
+    SPVM_COMPILER_add_basic_types(compiler);
+    
+    SPVM_COMPILER_set_default_loaded_module_files(compiler);
+  }
+  
+  SPVM_COMPILER_use_default_loaded_modules(compiler);
+  
+  if (basic_type_name) {
+    SPVM_STRING* basic_type_name_string = SPVM_STRING_new(compiler, basic_type_name, strlen(basic_type_name));
+    basic_type_name = basic_type_name_string->value;
+    const char* start_file = SPVM_COMPILER_get_start_file(compiler);
+    int32_t start_line = SPVM_COMPILER_get_start_line(compiler);
+    
+    SPVM_COMPILER_use(compiler, basic_type_name, start_file, start_line);
+  }
+  
+#ifdef SPVM_DEBUG_YACC
+  SPVM_yydebug = 1;
+#else
+  SPVM_yydebug = 0;
+#endif
+
+  compiler->end_of_file = 1;
+  
+  int32_t yyparse_error_id = SPVM_yyparse(compiler);
+  
+  SPVM_COMPILER_assert_check_basic_type_ids(compiler);
+  
+  int32_t error_id = 0;
+  
+  if (yyparse_error_id) {
+    error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_CLASS;
+  }
+  else {
+    if (SPVM_COMPILER_get_error_messages_length(compiler) > 0) {
+      error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_CLASS;
+    }
+    else {
+      SPVM_CHECK_check(compiler);
+      if (SPVM_COMPILER_get_error_messages_length(compiler) > 0) {
+        error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_CLASS;
+      }
+      else {
+        int32_t build_opcode_list_start_memory_blocks_count_tmp = compiler->current_each_compile_allocator->memory_blocks_count_tmp;
+        SPVM_OPCODE_BUILDER_build_opcode_list(compiler);
+        assert(compiler->current_each_compile_allocator->memory_blocks_count_tmp == build_opcode_list_start_memory_blocks_count_tmp);
+        assert(SPVM_COMPILER_get_error_messages_length(compiler) == 0);
+      }
+    }
+  }
+  
+  SPVM_COMPILER_free_memory_tmp_each_compile(compiler);
+  
+  assert(compiler->current_each_compile_allocator->memory_blocks_count_tmp == compile_start_memory_blocks_count_tmp);
+  
+  if (error_id) {
+    SPVM_ALLOCATOR_free(compiler->current_each_compile_allocator);
+    compiler->current_each_compile_allocator = NULL;
+  }
+  else {
+    SPVM_LIST_push(compiler->each_compile_allocators, compiler->current_each_compile_allocator);
+    compiler->current_each_compile_allocator = NULL;
+    
+    SPVM_COMPILER_build_runtime(compiler);
+  }
+  
+  return error_id;
+}
+
+int32_t SPVM_COMPILER_clear_error_messages(SPVM_COMPILER* compiler) {
+  
+  SPVM_LIST* error_messages = compiler->error_messages;
+  
+  for (int32_t i = 0; i < error_messages->length; i++) {
+    const char* error_message = SPVM_LIST_get(error_messages, i);
+    
+    SPVM_ALLOCATOR_free_memory_block_tmp(compiler->error_message_allocator, (void*)error_message);
+    error_message = NULL;
+  }
+  
+  error_messages->length = 0;
+}
+
 SPVM_MODULE_FILE* SPVM_COMPILER_get_module_file(SPVM_COMPILER* compiler, const char* module_name) {
   SPVM_MODULE_FILE* module_file = SPVM_HASH_get(compiler->module_file_symtable, module_name, strlen(module_name));
   
@@ -156,16 +284,6 @@ void SPVM_COMPILER_use(SPVM_COMPILER* compiler, const char* basic_type_name, con
   SPVM_LIST_push(compiler->op_use_stack, op_use);
 }
 
-void SPVM_COMPILER_check_basic_type_id(SPVM_COMPILER* compiler, int32_t basic_type_id) {
-  SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
-  const char* basic_type_name_expected = SPVM_BASIC_TYPE_get_basic_type_name(compiler, basic_type_id);
-  
-  if (strcmp(basic_type->name, basic_type_name_expected) != 0) {
-    fprintf(stderr, "[Unexpected Error]Basic Type ID:%d, Basic Type Name:%s, Expected Basic Type Name: %s\n", basic_type_id, basic_type->name, basic_type_name_expected);
-    assert(0);
-  }
-}
-
 const char* SPVM_COMPILER_get_runtime_name(SPVM_HASH* runtime_constant_string_symtable, const char* name) {
   
   SPVM_RUNTIME_STRING* string = SPVM_HASH_get(runtime_constant_string_symtable, name, strlen(name));
@@ -175,7 +293,7 @@ const char* SPVM_COMPILER_get_runtime_name(SPVM_HASH* runtime_constant_string_sy
   return new_name;
 }
 
-int32_t SPVM_COMPILER_use_default_loaded_classes(SPVM_COMPILER* compiler) {
+int32_t SPVM_COMPILER_use_default_loaded_modules(SPVM_COMPILER* compiler) {
   // Use automatically loaded classes
   SPVM_COMPILER_use(compiler, "Byte", "Byte", 0);
   SPVM_COMPILER_use(compiler, "Short", "Short", 0);
@@ -191,283 +309,226 @@ int32_t SPVM_COMPILER_use_default_loaded_classes(SPVM_COMPILER* compiler) {
   SPVM_COMPILER_use(compiler, "Address", "Address", 0);
 }
 
-int32_t SPVM_COMPILER_compile(SPVM_COMPILER* compiler, const char* basic_type_name) {
-  
-  compiler->basic_types_base_id = compiler->basic_types->length;
-  
-  if (compiler->basic_types->length == 0) {
-    // Add basic types
-    SPVM_COMPILER_add_basic_types(compiler);
-    
-    // Add Bool module file
-    {
-      SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
-      module_file->module_name = "Bool";
-      module_file->rel_file = "Bool.spvm";
-      module_file->content = "class Bool {\n  INIT {\n    $TRUE = new Bool;\n    $TRUE->{value} = 1;\n    $FALSE = new Bool;\n    $FALSE->{value} = 0;\n  }\n  \n  our $TRUE : ro Bool;\n  our $FALSE : ro Bool;\n  has value : ro int;\n}";
-      module_file->content_length = strlen(module_file->content);
-      SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
-    }
-    
-    // Add Error module file
-    {
-      SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
-      module_file->module_name = "Error";
-      module_file->rel_file = "Error.spvm";
-      module_file->content = "class Error;";
-      module_file->content_length = strlen(module_file->content);
-      SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
-    }
-    
-    // Add Error::System module file
-    {
-      SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
-      module_file->module_name = "Error::System";
-      module_file->rel_file = "Error/System.spvm";
-      module_file->content = "class Error::System extends Error;";
-      module_file->content_length = strlen(module_file->content);
-      SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
-    }
-    
-    // Add Error::NotSupported module file
-    {
-      SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
-      module_file->module_name = "Error::NotSupported";
-      module_file->rel_file = "Error/NotSupported.spvm";
-      module_file->content = "class Error::NotSupported extends Error;";
-      module_file->content_length = strlen(module_file->content);
-      SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
-    }
-    
-    // Add Byte module file
-    {
-      SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
-      module_file->module_name = "Byte";
-      module_file->rel_file = "Byte.spvm";
-      module_file->content = "class Byte {\n  has value : ro byte;\n  static method new : Byte ($value : int) {\n    my $self = new Byte;\n    $self->{value} = (byte)$value;\n    return $self;\n  }\n}";
-      module_file->content_length = strlen(module_file->content);
-      SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
-    }
-    
-    // Add Short module file
-    {
-      SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
-      module_file->module_name = "Short";
-      module_file->rel_file = "Short.spvm";
-      module_file->content = "class Short {\n  has value : ro short;\n  static method new : Short ($value : int) {\n    my $self = new Short;\n    $self->{value} = (short)$value;\n    return $self;\n  }\n}";
-      module_file->content_length = strlen(module_file->content);
-      SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
-    }
-    
-    // Add Int module file
-    {
-      SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
-      module_file->module_name = "Int";
-      module_file->rel_file = "Int.spvm";
-      module_file->content = "class Int {\n  has value : ro int;\n  static method new : Int ($value : int) {\n    my $self = new Int;\n    $self->{value} = $value;\n    return $self;\n  }\n}";
-      module_file->content_length = strlen(module_file->content);
-      SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
-    }
-    
-    // Add Long module file
-    {
-      SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
-      module_file->module_name = "Long";
-      module_file->rel_file = "Long.spvm";
-      module_file->content = "class Long {\n  has value : ro long;\n  static method new : Long ($value : long) {\n    my $self = new Long;\n    $self->{value} = $value;\n    return $self;\n  }\n}";
-      module_file->content_length = strlen(module_file->content);
-      SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
-    }
-    
-    // Add Float module file
-    {
-      SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
-      module_file->module_name = "Float";
-      module_file->rel_file = "Float.spvm";
-      module_file->content = "class Float {\n  has value : ro float;\n  static method new : Float ($value : float) {\n    my $self = new Float;\n    $self->{value} = $value;\n    return $self;\n  }\n}";
-      module_file->content_length = strlen(module_file->content);
-      SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
-    }
-    
-    // Add Double module file
-    {
-      SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
-      module_file->module_name = "Double";
-      module_file->rel_file = "Double.spvm";
-      module_file->content = "class Double {\n  has value : ro double;\n  static method new : Double ($value : double) {\n    my $self = new Double;\n    $self->{value} = $value;\n    return $self;\n  }\n}";
-      module_file->content_length = strlen(module_file->content);
-      SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
-    }
-    
-    // Add CommandInfo module file
-    {
-      SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
-      module_file->module_name = "CommandInfo";
-      module_file->rel_file = "CommandInfo.spvm";
-      module_file->content = "class CommandInfo {\n  our $PROGRAM_NAME : ro string;\n  our $ARGV : ro string[];\n  our $BASE_TIME : ro long;\n  }";
-      module_file->content_length = strlen(module_file->content);
-      SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
-    }
-    
-    // Add Address module file
-    {
-      SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
-      module_file->module_name = "Address";
-      module_file->rel_file = "Address.spvm";
-      module_file->content = "class Address : pointer {\n  static method new : Address () {\n    my $self = new Address;\n    return $self;\n  }\n}";
-      module_file->content_length = strlen(module_file->content);
-      SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
-    }
-  }
-  
-  const char* start_file = SPVM_COMPILER_get_start_file(compiler);
-  int32_t start_line = SPVM_COMPILER_get_start_line(compiler);
-  
-  //yacc/bison debug mode. The default is off.
-  SPVM_yydebug = 0;
-  
-#ifdef SPVM_DEBUG_YACC
-  // Turn on yacc/bison debug mode
-  SPVM_yydebug = 1;
-#endif
-
-  compiler->parse_not_started = 1;
- 
-  // Initialize error messages
-  compiler->error_messages = SPVM_LIST_new_list_permanent(compiler->allocator, 0);
-  
-  int32_t error_id = 0;
-  
-  int32_t compile_start_memory_blocks_count_tmp = compiler->allocator->memory_blocks_count_tmp;
-  
-  compiler->ops = SPVM_LIST_new(compiler->allocator, 0, SPVM_ALLOCATOR_C_ALLOC_TYPE_TMP);
-  compiler->op_use_stack = SPVM_LIST_new(compiler->allocator, 0, SPVM_ALLOCATOR_C_ALLOC_TYPE_TMP);
-  compiler->op_types = SPVM_LIST_new(compiler->allocator, 0, SPVM_ALLOCATOR_C_ALLOC_TYPE_TMP);
-  
-  SPVM_COMPILER_use_default_loaded_classes(compiler);
-  
-  // Use the class that is specified at the argument
-  if (basic_type_name) {
-    SPVM_STRING* basic_type_name_string = SPVM_STRING_new(compiler, basic_type_name, strlen(basic_type_name));
-    basic_type_name = basic_type_name_string->value;
-    SPVM_COMPILER_use(compiler, basic_type_name, start_file, start_line);
-  }
-  
-  /* Tokenize and Parse */
-  int32_t parse_error_flag = SPVM_yyparse(compiler);
-  
+void SPVM_COMPILER_set_default_loaded_module_files(SPVM_COMPILER* compiler) {
+  // Add Bool module file
   {
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_UNKNOWN);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_UNDEF);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_VOID);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_BYTE);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_SHORT);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_INT);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_LONG);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_FLOAT);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_DOUBLE);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_STRING);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_ANY_OBJECT);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_BYTE_CLASS);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_SHORT_CLASS);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_INT_CLASS);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_LONG_CLASS);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_FLOAT_CLASS);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_DOUBLE_CLASS);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_BOOL_CLASS);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_CLASS);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_NOT_SUPPORTED_CLASS);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_COMMAND_INFO_CLASS);
-    SPVM_COMPILER_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_ADDRESS_CLASS);
+    SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
+    module_file->module_name = "Bool";
+    module_file->rel_file = "Bool.spvm";
+    module_file->content = "class Bool {\n  INIT {\n    $TRUE = new Bool;\n    $TRUE->{value} = 1;\n    $FALSE = new Bool;\n    $FALSE->{value} = 0;\n  }\n  \n  our $TRUE : ro Bool;\n  our $FALSE : ro Bool;\n  has value : ro int;\n}";
+    module_file->content_length = strlen(module_file->content);
+    SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
   }
   
-  if (parse_error_flag) {
-    error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_CLASS;
-  }
-  else {
-    if (SPVM_COMPILER_get_error_messages_length(compiler) > 0) {
-      error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_CLASS;
-    }
-    else {
-      // Check syntax
-      SPVM_CHECK_check(compiler);
-      if (SPVM_COMPILER_get_error_messages_length(compiler) > 0) {
-        error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_CLASS;
-      }
-      else {
-        // Build operation code
-        int32_t build_opcode_list_start_memory_blocks_count_tmp = compiler->allocator->memory_blocks_count_tmp;
-        SPVM_OPCODE_BUILDER_build_opcode_list(compiler);
-        assert(compiler->allocator->memory_blocks_count_tmp == build_opcode_list_start_memory_blocks_count_tmp);
-        if (SPVM_COMPILER_get_error_messages_length(compiler) > 0) {
-          error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_CLASS;
-        }
-      }
-    }
+  // Add Error module file
+  {
+    SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
+    module_file->module_name = "Error";
+    module_file->rel_file = "Error.spvm";
+    module_file->content = "class Error;";
+    module_file->content_length = strlen(module_file->content);
+    SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
   }
   
-  // Cleanup ops
+  // Add Error::System module file
+  {
+    SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
+    module_file->module_name = "Error::System";
+    module_file->rel_file = "Error/System.spvm";
+    module_file->content = "class Error::System extends Error;";
+    module_file->content_length = strlen(module_file->content);
+    SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
+  }
+  
+  // Add Error::NotSupported module file
+  {
+    SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
+    module_file->module_name = "Error::NotSupported";
+    module_file->rel_file = "Error/NotSupported.spvm";
+    module_file->content = "class Error::NotSupported extends Error;";
+    module_file->content_length = strlen(module_file->content);
+    SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
+  }
+  
+  // Add Byte module file
+  {
+    SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
+    module_file->module_name = "Byte";
+    module_file->rel_file = "Byte.spvm";
+    module_file->content = "class Byte {\n  has value : ro byte;\n  static method new : Byte ($value : int) {\n    my $self = new Byte;\n    $self->{value} = (byte)$value;\n    return $self;\n  }\n}";
+    module_file->content_length = strlen(module_file->content);
+    SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
+  }
+  
+  // Add Short module file
+  {
+    SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
+    module_file->module_name = "Short";
+    module_file->rel_file = "Short.spvm";
+    module_file->content = "class Short {\n  has value : ro short;\n  static method new : Short ($value : int) {\n    my $self = new Short;\n    $self->{value} = (short)$value;\n    return $self;\n  }\n}";
+    module_file->content_length = strlen(module_file->content);
+    SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
+  }
+  
+  // Add Int module file
+  {
+    SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
+    module_file->module_name = "Int";
+    module_file->rel_file = "Int.spvm";
+    module_file->content = "class Int {\n  has value : ro int;\n  static method new : Int ($value : int) {\n    my $self = new Int;\n    $self->{value} = $value;\n    return $self;\n  }\n}";
+    module_file->content_length = strlen(module_file->content);
+    SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
+  }
+  
+  // Add Long module file
+  {
+    SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
+    module_file->module_name = "Long";
+    module_file->rel_file = "Long.spvm";
+    module_file->content = "class Long {\n  has value : ro long;\n  static method new : Long ($value : long) {\n    my $self = new Long;\n    $self->{value} = $value;\n    return $self;\n  }\n}";
+    module_file->content_length = strlen(module_file->content);
+    SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
+  }
+  
+  // Add Float module file
+  {
+    SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
+    module_file->module_name = "Float";
+    module_file->rel_file = "Float.spvm";
+    module_file->content = "class Float {\n  has value : ro float;\n  static method new : Float ($value : float) {\n    my $self = new Float;\n    $self->{value} = $value;\n    return $self;\n  }\n}";
+    module_file->content_length = strlen(module_file->content);
+    SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
+  }
+  
+  // Add Double module file
+  {
+    SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
+    module_file->module_name = "Double";
+    module_file->rel_file = "Double.spvm";
+    module_file->content = "class Double {\n  has value : ro double;\n  static method new : Double ($value : double) {\n    my $self = new Double;\n    $self->{value} = $value;\n    return $self;\n  }\n}";
+    module_file->content_length = strlen(module_file->content);
+    SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
+  }
+  
+  // Add CommandInfo module file
+  {
+    SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
+    module_file->module_name = "CommandInfo";
+    module_file->rel_file = "CommandInfo.spvm";
+    module_file->content = "class CommandInfo {\n  our $PROGRAM_NAME : ro string;\n  our $ARGV : ro string[];\n  our $BASE_TIME : ro long;\n  }";
+    module_file->content_length = strlen(module_file->content);
+    SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
+  }
+  
+  // Add Address module file
+  {
+    SPVM_MODULE_FILE* module_file = SPVM_MODULE_FILE_new(compiler);
+    module_file->module_name = "Address";
+    module_file->rel_file = "Address.spvm";
+    module_file->content = "class Address : pointer {\n  static method new : Address () {\n    my $self = new Address;\n    return $self;\n  }\n}";
+    module_file->content_length = strlen(module_file->content);
+    SPVM_COMPILER_set_module_file(compiler, module_file->module_name, module_file);
+  }
+}
+
+void SPVM_COMPILER_assert_check_basic_type_ids(SPVM_COMPILER* compiler) {
+  
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_UNKNOWN);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_UNDEF);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_VOID);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_BYTE);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_SHORT);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_INT);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_LONG);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_FLOAT);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_DOUBLE);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_STRING);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_ANY_OBJECT);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_BYTE_CLASS);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_SHORT_CLASS);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_INT_CLASS);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_LONG_CLASS);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_FLOAT_CLASS);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_DOUBLE_CLASS);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_BOOL_CLASS);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_CLASS);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_NOT_SUPPORTED_CLASS);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_COMMAND_INFO_CLASS);
+  SPVM_COMPILER_assert_check_basic_type_id(compiler, SPVM_NATIVE_C_BASIC_TYPE_ID_ADDRESS_CLASS);
+}
+
+void SPVM_COMPILER_assert_check_basic_type_id(SPVM_COMPILER* compiler, int32_t basic_type_id) {
+  SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
+  const char* basic_type_name_expected = SPVM_BASIC_TYPE_get_basic_type_name(compiler, basic_type_id);
+  
+  if (strcmp(basic_type->name, basic_type_name_expected) != 0) {
+    fprintf(stderr, "[Unexpected Error]Basic Type ID:%d, Basic Type Name:%s, Expected Basic Type Name: %s\n", basic_type_id, basic_type->name, basic_type_name_expected);
+    assert(0);
+  }
+}
+
+void SPVM_COMPILER_free_memory_tmp_each_compile(SPVM_COMPILER* compiler) {
   for (int32_t i = 0; i < compiler->ops->length; i++) {
     SPVM_OP* op = SPVM_LIST_get(compiler->ops, i);
     int32_t op_id = op->id;
     switch(op_id) {
       case SPVM_OP_C_ID_BLOCK: {
-        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->allocator, op->uv.block);
+        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->current_each_compile_allocator, op->uv.block);
         break;
       }
       case SPVM_OP_C_ID_ATTRIBUTE: {
-        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->allocator, op->uv.attribute);
+        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->current_each_compile_allocator, op->uv.attribute);
         break;
       }
       case SPVM_OP_C_ID_USE: {
         SPVM_USE* use = op->uv.use;
         use->alias_name = NULL;
-        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->allocator, use);
+        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->current_each_compile_allocator, use);
         break;
       }
       case SPVM_OP_C_ID_ALLOW: {
         SPVM_ALLOW* allow = op->uv.allow;
-        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->allocator, allow);
+        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->current_each_compile_allocator, allow);
         break;
       }
       case SPVM_OP_C_ID_INTERFACE: {
         SPVM_INTERFACE* interface = op->uv.interface;
-        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->allocator, interface);
+        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->current_each_compile_allocator, interface);
         break;
       }
       case SPVM_OP_C_ID_CLASS_VAR_ACCESS: {
         SPVM_CLASS_VAR_ACCESS* class_var_access = op->uv.class_var_access;
         class_var_access->op_name = NULL;
         class_var_access->class_var = NULL;
-        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->allocator, class_var_access);
+        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->current_each_compile_allocator, class_var_access);
         break;
       }
       case SPVM_OP_C_ID_CONSTANT: {
         SPVM_CONSTANT* constant = op->uv.constant;
         constant->op_constant = NULL;
         constant->type = NULL;
-        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->allocator, constant);
+        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->current_each_compile_allocator, constant);
         break;
       }
       case SPVM_OP_C_ID_ARRAY_FIELD_ACCESS: {
         SPVM_ARRAY_FIELD_ACCESS* array_field_access = op->uv.array_field_access;
         array_field_access->field = NULL;
-        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->allocator, array_field_access);
+        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->global_allocator, array_field_access);
         break;
       }
       case SPVM_OP_C_ID_FIELD_ACCESS: {
         SPVM_FIELD_ACCESS* field_access = op->uv.field_access;
         field_access->op_name = NULL;
         field_access->field = NULL;
-        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->allocator, field_access);
+        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->current_each_compile_allocator, field_access);
         break;
       }
       case SPVM_OP_C_ID_CALL_METHOD: {
         SPVM_CALL_METHOD* call_method = op->uv.call_method;
         call_method->op_name = NULL;
         call_method->method = NULL;
-        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->allocator, call_method);
+        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->current_each_compile_allocator, call_method);
         break;
       }
       case SPVM_OP_C_ID_VAR: {
@@ -476,29 +537,26 @@ int32_t SPVM_COMPILER_compile(SPVM_COMPILER* compiler, const char* basic_type_na
         var->name = NULL;
         var->var_decl = NULL;
         var->call_method = NULL;
-        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->allocator, var);
+        SPVM_ALLOCATOR_free_memory_block_tmp(compiler->current_each_compile_allocator, var);
         break;
       }
-      case SPVM_OP_C_ID_MY: {
+      case SPVM_OP_C_ID_VAR_DECL: {
         SPVM_VAR_DECL* var_decl = op->uv.var_decl;
-        if (!var_decl->is_permanent) {
+        if (!var_decl->is_arg) {
           var_decl->op_var_decl = NULL;
           var_decl->type = NULL;
           var_decl->var = NULL;
-          SPVM_ALLOCATOR_free_memory_block_tmp(compiler->allocator, var_decl);
+          SPVM_ALLOCATOR_free_memory_block_tmp(compiler->current_each_compile_allocator, var_decl);
         }
         break;
       }
     }
-    SPVM_ALLOCATOR_free_memory_block_tmp(compiler->allocator, op);
+    SPVM_ALLOCATOR_free_memory_block_tmp(compiler->current_each_compile_allocator, op);
   }
   
-  // Clear unused pointers
   for (int32_t basic_type_id = compiler->basic_types_base_id; basic_type_id < compiler->basic_types->length; basic_type_id++) {
     SPVM_BASIC_TYPE* basic_type = SPVM_LIST_get(compiler->basic_types, basic_type_id);
     
-    SPVM_LIST_free(basic_type->allows);
-    
     SPVM_LIST* methods = basic_type->methods;
     {
       int32_t method_index;
@@ -511,23 +569,11 @@ int32_t SPVM_COMPILER_compile(SPVM_COMPILER* compiler, const char* basic_type_na
     }
   }
   
-  // Free
-  SPVM_LIST_free(compiler->op_use_stack);
-  compiler->op_use_stack = NULL;
+  compiler->op_use_stack->length = 0;
   
-  SPVM_LIST_free(compiler->op_types);
-  compiler->op_types = NULL;
+  compiler->op_types->length = 0;
   
-  SPVM_LIST_free(compiler->ops);
-  compiler->ops = NULL;
-  
-  assert(compiler->allocator->memory_blocks_count_tmp == compile_start_memory_blocks_count_tmp);
-  
-  if (!error_id) {
-    SPVM_COMPILER_build_runtime(compiler);
-  }
-  
-  return error_id;
+  compiler->ops->length = 0;
 }
 
 SPVM_RUNTIME* SPVM_COMPILER_build_runtime(SPVM_COMPILER* compiler) {
@@ -745,34 +791,34 @@ SPVM_RUNTIME* SPVM_COMPILER_get_runtime(SPVM_COMPILER* compiler) {
   return compiler->runtime;
 }
 
-void SPVM_COMPILER_error(SPVM_COMPILER* compiler, const char* message_template, ...) {
+void SPVM_COMPILER_error(SPVM_COMPILER* compiler, const char* error_message_template, ...) {
   
-  int32_t message_length = 0;
+  int32_t error_message_length = 0;
   
   // Message template
-  int32_t message_template_length = (int32_t)strlen(message_template);
+  int32_t error_message_template_length = (int32_t)strlen(error_message_template);
   
   va_list args;
-  va_start(args, message_template);
+  va_start(args, error_message_template);
   
-  message_length += message_template_length;
+  error_message_length += error_message_template_length;
   
   // Argument count
-  char* found_ptr = (char*)message_template;
+  char* found_ptr = (char*)error_message_template;
   while (1) {
     found_ptr = strchr(found_ptr, '%');
     if (found_ptr) {
       if (*(found_ptr + 1) == 'c') {
         int arg = va_arg(args, int);
-        message_length++;
+        error_message_length++;
       }
       else if (*(found_ptr + 1) == 's') {
         char* arg = va_arg(args, char*);
-        message_length += strlen(arg);
+        error_message_length += strlen(arg);
       }
       else if (*(found_ptr + 1) == 'd') {
         (void) va_arg(args, int);
-        message_length += 30;
+        error_message_length += 30;
       }
       else if (*(found_ptr + 1) == '%') {
         // Nothing
@@ -789,33 +835,13 @@ void SPVM_COMPILER_error(SPVM_COMPILER* compiler, const char* message_template,
   }
   va_end(args);
   
-  char* message = SPVM_ALLOCATOR_alloc_memory_block_permanent(compiler->allocator, message_length + 1);
+  char* error_message = SPVM_ALLOCATOR_alloc_memory_block_tmp(compiler->error_message_allocator, error_message_length + 1);
   
-  va_start(args, message_template);
-  vsprintf(message, message_template, args);
+  va_start(args, error_message_template);
+  vsprintf(error_message, error_message_template, args);
   va_end(args);
   
-  SPVM_LIST_push(compiler->error_messages, message);
-}
-
-void SPVM_COMPILER_free(SPVM_COMPILER* compiler) {
-  
-  const char* start_file = SPVM_COMPILER_get_start_file(compiler);
-  
-  if (start_file) {
-    SPVM_ALLOCATOR_free_memory_block_tmp(compiler->allocator, (void*)start_file);
-  }
-  
-  SPVM_COMPILER_clear_include_dirs(compiler);
-  
-  if (compiler->runtime) {
-    SPVM_RUNTIME_free(compiler->runtime);
-    compiler->runtime = NULL;
-  }
-  
-  // Free allocator
-  SPVM_ALLOCATOR_free(compiler->allocator);
-  compiler->allocator = NULL;
+  SPVM_LIST_push(compiler->error_messages, error_message);
 }
 
 const char* SPVM_COMPILER_get_start_file(SPVM_COMPILER* compiler) {
@@ -825,14 +851,14 @@ const char* SPVM_COMPILER_get_start_file(SPVM_COMPILER* compiler) {
 void SPVM_COMPILER_set_start_file(SPVM_COMPILER* compiler, const char* start_file) {
   
   if (compiler->start_file) {
-    SPVM_ALLOCATOR_free_memory_block_tmp(compiler->allocator, (void*)compiler->start_file);
+    SPVM_ALLOCATOR_free_memory_block_tmp(compiler->global_allocator, (void*)compiler->start_file);
     compiler->start_file = NULL;
   }
   
   char* compiler_start_file = NULL;
   if (start_file) {
     int32_t start_file_length = strlen(start_file);
-    compiler_start_file = SPVM_ALLOCATOR_alloc_memory_block_tmp(compiler->allocator, start_file_length + 1);
+    compiler_start_file = SPVM_ALLOCATOR_alloc_memory_block_tmp(compiler->global_allocator, start_file_length + 1);
     memcpy(compiler_start_file, start_file, start_file_length);
   }
   
@@ -855,7 +881,7 @@ int32_t SPVM_COMPILER_get_include_dirs_length(SPVM_COMPILER* compiler) {
 
 void SPVM_COMPILER_add_include_dir(SPVM_COMPILER* compiler, const char* include_dir) {  
   int32_t include_dir_length = strlen(include_dir);
-  char* compiler_include_dir = SPVM_ALLOCATOR_alloc_memory_block_tmp(compiler->allocator, include_dir_length + 1);
+  char* compiler_include_dir = SPVM_ALLOCATOR_alloc_memory_block_tmp(compiler->global_allocator, include_dir_length + 1);
   memcpy(compiler_include_dir, include_dir, include_dir_length);
   SPVM_LIST_push(compiler->include_dirs, (void*)compiler_include_dir);
 }
@@ -865,7 +891,7 @@ void SPVM_COMPILER_clear_include_dirs(SPVM_COMPILER* compiler) {
   
   for (int32_t i = 0; i < include_dirs_length; i++) {
     const char* include_dir = SPVM_COMPILER_get_include_dir(compiler, i);
-    SPVM_ALLOCATOR_free_memory_block_tmp(compiler->allocator, (void*)include_dir);
+    SPVM_ALLOCATOR_free_memory_block_tmp(compiler->global_allocator, (void*)include_dir);
     include_dir = NULL;
   }
   
